import uuid
import os
import os.path
import shutil
from datetime import date, timedelta

from jinja2 import Template

from .headers import URL, Inline

class Page:
    template = """<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>{{ title }}</title>

    <!-- Bootstrap -->
    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.4/css/bootstrap.min.css">

    <!-- HTML5 shim and Respond.js for IE8 support of HTML5 elements and media queries -->
    <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
    <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/html5shiv/3.7.2/html5shiv.min.js"></script>
      <script src="https://oss.maxcdn.com/respond/1.4.2/respond.min.js"></script>
    <![endif]-->
  <style>
  @media (min-width: 768px) {
      .container {
          max-width: 730px;
          margin-left: auto;
          margin-right: auto;
      }
  }
  
  body { padding-top: 70px; }
  
  @media screen and (max-width: 768px) {
      body { padding-top: 100px; }
  }
  
  section {
      margin-top:    5px;
      margin-bottom: 15px;
  }
  </style>
  {% for style in css %}
  {{ style }}
  {% endfor %}
  {% for script in js %}
  {{ script }}
  {% endfor %}
  <script>
  $(document).ready(function() {
  {% for script in on_ready %}
  {{ script }}
  {% endfor %}
  });
  </script>
  </head>
  <body>
    <div class="navbar navbar-default navbar-fixed-top" role="navigation">
      <div class="container-fluid">
        <div class="navbar-header">
          <a class="navbar-brand" href="#">{{title}}</a>
        </div>
        <div class="nav navbar-text navbar-right">Uploaded on {{date}}, expire on {{expire}}</div>
      </div>
    </div>
    <div class="container">
{% for section in sections %}
{{ section }}
{% endfor %}
    </div>
  <footer>Powered by <a href="http://hub.darcs.net/oschwand/postcard">Postcard</a></footer>
  </body>
</html>
"""
    
    def __init__(self, sections, expire=90):
        self.uuid      = str(uuid.uuid4())
        self._sections = sections
        self._today    = date.today()
        self.expire    = self._today + timedelta(days=expire)
    
    def title(self):
        return self._sections[0].title()
    
    def summary(self, buffer):
        for section in self._sections:
            print(section.summary(), file=buffer)
    
    def prepare(self, configuration):
        for section in self._sections:
            section.prepare(configuration.get(section.name, {}))
    
    def html(self, buffer):
        template = Template(self.template)
        
        title    = self.title()
        sections = [ section.html() for section in self._sections ]
        
        js = []
        for section in self._sections:
            for script in section.js():
                if isinstance(script, URL):
                    line = "<script src=\"{url}\"></script>".format(url=script)
                elif isinstance(script, Inline):
                    line = "<script>{code}</script>".format(code=script)
                if line not in js:
                    js.append(line)
        
        css = []
        for section in self._sections:
            for style in section.css():
                if isinstance(style, URL):
                    line = "<link rel=\"stylesheet\" href=\"{url}\" type=\"text/css\"/>".format(url=style)
                elif isinstance(style, Inline):
                    line = "<style>{code}</style>".format(code=style)
                if line not in css:
                    css.append(line)
        
        on_ready = []
        for section in self._sections:
            for script in section.on_ready():
                on_ready.append(script)
        
        html = template.render(title=title,
                               sections=sections,
                               css=css,
                               js=js,
                               date=self._today,
                               expire=self.expire,
                               on_ready=on_ready,
                           )
        
        print(html, file=buffer)
    
    def attachments(self):
        for section in self._sections:
            for attachment in section.attachments():
                yield attachment

#"//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.1.1/css/bootstrap.css"
# a.gallery img {
#     margin-top:    3px;
# }
#                 """,
#         ],
#             "title": "",
#         }
        
#         self.tmpdir  = tmpdir
#         self.pagedir = os.path.join(tmpdir, self.uuid)
#         os.mkdir(self.pagedir)
#         os.chdir(self.pagedir)
        #         self._index = open(os.path.join(self.tmpdir, self.uuid, "index.html"), "w")
    
#     def clean(self):
#         shutil.rmtree(self.tmpdir)
#     def ready(self, line):
#         if line not in self.headers["ready"]:
#             self.headers["ready"].append(line)
    
#         """.format(title=title, date=self._today.isoformat()))
#         index.write("<div class=\"container\">\n")
    
